function [invtidx,eigvalue, modulus, matrices,A,B,C,D] = invertest_abcd(A,C0,M,neps,nerror,obs,shock)

% Poor Man's Invertibility Test from Fernndez-Villaverde,
% Rubio-Ramrez, Sargent, and Watson (2007), AER.
% Assumes state-space form:
%   State: x(t+1)= A*x(t) + B*w(t+1)
%   Observation: y(t+1) = C*x(t) + D*w(t+1)
% Tests that the eigenvalues of A-B*inv(D)*C are stricly less than
% one in modulus.
% Last updated: May 7, 2008
%
% INPUT:
% A:        "G" matrix from gensys, nvar x nvar matrix where nvar is
%           the number of variables in the model.
% C0:       "C" matrix from gensys.
% M:        "impact" matrix from gensys, either nvar x nvar matrix or 
%           nvar x neps matrix 
% neps:     Number of structural shocks in model
% nerror:   Number of measurement errors.
% obs:      (neps + nerror) x 1 vector that denotes which variables of the
%           G matrix are the observables.  For instance, if the first two  
%           and seventh variables of the G matrix are observable, then 
%           obs = [1; 2; 7].  Observables with measurement error are
%           assumed to be the last elements of the vector obs.
% shock:    If the size of M is not nvar x neps, then this vector must
%           be specified; otherwise, it is not needed.  neps x 1 vector 
%           that denotes which columns of M correspond to shocks.  For
%           instance, if the first 3 columns represent the structural
%           shocks, then shock = [1; 2; 3].
%           
% OUTPUT:
% eigvalue: The eigenvalues of A - B*inv(D)*C.
% modulus:  The absolute value of the eigenvalues.
% matrices: The A,B,C,and D matrices.


[nvar,nshock] = size(M);    
nobs = length(obs);       % number of observables


if sum(C0 ~= 0) ~= 0   % In this case, a constant must be added to the state vector
    C0 = [C0; 1];
    A = [A; zeros(1,nvar)];
    A = [A C0];
    M = [M; zeros(1,nshock)];
    B = zeros(nvar+1,neps+nerror);
    C = zeros(nobs,nvar+1);
    D = zeros(nobs,neps+nerror);
    eigvalue = zeros(nvar+1,1);
    modulus = zeros(nvar+1,1);
else
    B = zeros(nvar,neps+nerror);
    C = zeros(nobs,nvar);
    D = zeros(nobs,neps+nerror);
    eigvalue = zeros(nvar,1);
    modulus = zeros(nvar,1);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%        Check for problems         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nargin < 7
    if nshock ~= neps
        disp('Warning: M matrix is not nvar x neps.')
        disp('Please specify "shock", the specific columns in M containing')
        disp('the structural shocks to proceed.')
        return
    else
        shock = (1:neps)'; 
    end
end
if neps+nerror < nobs
    disp('Error: Too many observables.  Observables must equal number of shocks.')
    disp('Add measurement errors or reduce observables.')
    return
end
if neps+nerror > nobs
    disp('Error: Too few observables.  Observables must equal number of shocks.')
    disp('Reduce measurement errors/structural shocks or increase observables.')
    return
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Compute matrices B, C, and D    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for i = 1:neps
    value1 = shock(i,1);
    B(:,i) = M(:,value1);
end
for i = 1:nobs
    value1 = obs(i,1);
    C(i,:) = A(value1,:);
    for j = 1:neps
        value2 = shock(j,1);
        D(i,j) = M(value1,value2);
    end
end
if nerror ~= 0  % adds measurement error if present
    for k = 1:nerror
        D(neps+k,neps+k) = 1;
    end
end

matrices.A = A;
matrices.B = B;
matrices.C = C;
matrices.D = D;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute the condition: A-B*inv(D)*C %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
eigvalue = eig(A - B*inv(D)*C);
modulus = abs(eigvalue);
total = sum(modulus>1);

if total < 1
    invtidx=1;  
%    disp(' ')
    disp('Invertibility condition satisfied.')
%    disp(' ')
else
    invtidx=0;
%    disp(' ')
%    disp('These observables do not satisfy invertibility condition.')
%    fprintf(1,'%d eigenvalues greater than one in modulus.\n', total)
%    disp(' ')
end



           
  
